//
//  signViewController.m
//  PLOP examples
//
//  Created by Rainer Schaaf on 09.01.15.
//  Copyright (c) 2015 Rainer Schaaf. All rights reserved.
//

#import "signViewController.h"
#import "PLOP_ios/PLOP_objc.h"

@interface signViewController ()

@end

@implementation signViewController

@synthesize webView;


- (void)viewDidLoad {
    [super viewDidLoad];
    
    // sign the document
    [self sign];
    
    // display the signed document
    NSArray *dirPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                            NSUserDomainMask, YES);
    NSString *docsDir = dirPaths[0];
    NSURL *url = [NSURL fileURLWithPath:[docsDir stringByAppendingPathComponent:@"invoice-signed.pdf"]];
    NSMutableURLRequest *urlRequest = [NSMutableURLRequest requestWithURL:url];
    
    [webView loadRequest:urlRequest];
}

- (void) sign {
    
    PLOP *plop = [[PLOP alloc] init];
    if (!plop) {
        return [self displayError:@"PLOP out of memory error."];
    }
 
    // Basic signature options
    NSString *sign_opts =
        @"sigtype=cms "     // use sigtype=cades for PAdES signature
        @"ltv=try "         // use ltv=full to ensure LTV status
        @"engine=builtin digitalid={filename=demorsa2048.p12} password={demo} ";
    
    // input files (here located in the mainBundle)
    NSString *infile  = [[NSBundle mainBundle] pathForResource:@"invoice" ofType:@"pdf"];
    
    // outputfile (write it to the DocumentDirectory)
    NSArray *dirPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                            NSUserDomainMask, YES);
    NSString *docsDir = dirPaths[0];
    NSString *outfile = [docsDir stringByAppendingPathComponent:@"invoice-signed.pdf"];


    bool visualize = false;    // Set to "true" to enable signature visualization
    NSString *visfile = [[NSBundle mainBundle] pathForResource:@"signing_man_pdfa1a" ofType:@"pdf"];

    @try {
        NSInteger doc;
        NSInteger visdoc = -1;
        NSString *optlist = sign_opts;
        
        // look for resources in "resourcePath"
        [plop set_option:[NSString stringWithFormat:@"searchpath {%@} ", [[NSBundle mainBundle] resourcePath]]];
        
        // open unprotected input file without any password
        if ((doc = [plop open_document:infile optlist:@""]) == -1) {
            return [self displayError:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]]];
        }
        
        if (visualize){
            NSString *fieldoptlist;
            if ((visdoc = [plop open_document:visfile optlist:@""]) == -1) {
                return [self displayError:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]]];
            }
            fieldoptlist = [NSString stringWithFormat:@"field={visdoc=%ld rect={50 50 adapt adapt}} ", (long)visdoc];
            optlist = [NSString stringWithFormat:@"%@%@", optlist, fieldoptlist];
        }
        
        if ([plop prepare_signature:optlist] == -1) {
            return [self displayError:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]]];
        }
        
        if (([plop create_document:outfile optlist:[NSString stringWithFormat:@"input=%ld", (long)doc]]) == -1) {
            return [self displayError:[NSString stringWithFormat:@"Error: %@: %@", [plop get_apiname], [plop get_errmsg]]];
        }
        
        // close input and output files
        [plop close_document:doc optlist:@""];
        
        if (visualize){
            [plop close_document:visdoc optlist:@""];
        }
    }
    @catch (PLOPException *ex) {
        NSString *errorText = [NSString stringWithFormat:@"Error %ld in %@(): %@\n", (long)[ex get_errnum], [ex get_apiname], [ex get_errmsg]];
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"PLOP crashed" message:errorText
                                                           delegate:self cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
        [alertView show];
    }
    @catch (NSException *ex) {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:[ex name] message:[ex reason]
                                                           delegate:self cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
        [alertView show];
    }
    @finally {
        plop = nil;
    }
    return;
}

- (void) displayError: (NSString *) message {
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"PLOP error" message:message delegate:nil cancelButtonTitle:nil otherButtonTitles:@"ok", nil];
    [alertView show];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
